<?php

namespace app\admin\controller\device;

use app\common\controller\Backend;
use think\Db;

/**
 * 站点列管理
 *
 * @icon fa fa-circle-o
 */
class Site extends Backend
{

    /**
     * Site模型对象
     * @var \app\admin\model\device\Site
     */
    protected $model = null;
    protected $searchFields = 'id,agent_user_id,area_name';   //里面为搜索的字段
    protected $selectpageFields = "id,agent_user_id,area_name";

    public function _initialize()
    {
        parent::_initialize();
        $this->model = new \app\admin\model\device\Site;
        $this->Chargegroupmodel = new \app\admin\model\device\Chargegroup();
        $this->Chargedevicemodel = new \app\admin\model\device\Chargedevice();
        $this->view->assign("statusList", $this->model->getStatusList());
    }



    /**
     * 默认生成的控制器所继承的父类中有index/add/edit/del/multi五个基础方法、destroy/restore/recyclebin三个回收站方法
     * 因此在当前控制器中可不用编写增删改查的代码,除非需要自己控制这部分逻辑
     * 需要将application/admin/library/traits/Backend.php中对应的方法复制到当前控制器,然后进行修改
     */


    /**
     * 查看
     *
     * @return string|Json
     * @throws \think\Exception
     * @throws DbException
     */
    public function index()
    {
        //设置过滤方法
        $this->request->filter(['strip_tags', 'trim']);
        if (false === $this->request->isAjax()) {
            return $this->view->fetch();
        }
        //如果发送的来源是 Selectpage，则转发到 Selectpage
        if ($this->request->request('keyField')) {
            return $this->selectpage();
        }
        [$where, $sort, $order, $offset, $limit] = $this->buildparams();
        $list = $this->model
            ->with('users,area')
            ->where($where)
            ->order($sort, $order)
            ->paginate($limit);
        $result = ['total' => $list->total(), 'rows' => $list->items()];
        return json($result);
    }

    /**
     * 添加
     *
     * @return string
     * @throws \think\Exception
     */
    public function add()
    {
        if (false === $this->request->isPost()) {
            return $this->view->fetch();
        }
        $params = $this->request->post('row/a');
        if (empty($params)) {
            $this->error(__('Parameter %s can not be empty', ''));
        }
        $params = $this->preExcludeFields($params);

        if ($this->dataLimit && $this->dataLimitFieldAutoFill) {
            $params[$this->dataLimitField] = $this->auth->id;
        }
        $result = false;
        Db::startTrans();
        try {
            //是否采用模型验证
            if ($this->modelValidate) {
                $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.add' : $name) : $this->modelValidate;
                $this->model->validateFailException()->validate($validate);
            }
            $group_name = $this->Chargegroupmodel->where(['id' => $params['group_id']])->value('group_name');
            $params['area_name'] = $group_name;
            $result = $this->model->allowField(true)->save($params);
            Db::commit();
        } catch (ValidateException | PDOException | Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($result === false) {
            $this->error(__('No rows were inserted'));
        }
        $this->success();
    }

    /**
     * 编辑
     *
     * @param $ids
     * @return string
     * @throws DbException
     * @throws \think\Exception
     */
    public function edit($ids = null)
    {
        $row = $this->model->get($ids);
        if (!$row) {
            $this->error(__('No Results were found'));
        }
        $adminIds = $this->getDataLimitAdminIds();
        if (is_array($adminIds) && !in_array($row[$this->dataLimitField], $adminIds)) {
            $this->error(__('You have no permission'));
        }
        if (false === $this->request->isPost()) {
            $this->view->assign('row', $row);
            return $this->view->fetch();
        }
        $params = $this->request->post('row/a');
        if (empty($params)) {
            $this->error(__('Parameter %s can not be empty', ''));
        }
        $params = $this->preExcludeFields($params);
        $result = false;
        Db::startTrans();
        try {
            //是否采用模型验证
            if ($this->modelValidate) {
                $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.edit' : $name) : $this->modelValidate;
                $row->validateFailException()->validate($validate);
            }
            $group_name = $this->Chargegroupmodel->where(['id' => $params['group_id']])->value('group_name');
            $params['area_name'] = $group_name;
            $result = $row->allowField(true)->save($params);
            Db::commit();
        } catch (ValidateException | PDOException | Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if (false === $result) {
            $this->error(__('No rows were updated'));
        }
        $this->success();
    }
    /**
     * 授权设备
     *
     * @return string
     * @throws \think\Exception
     */
    public function authorizedevice()
    {
        if (false === $this->request->isPost()) {
            return $this->view->fetch();
        }
        $params = $this->request->post('row/a');
        if (empty($params)) {
            $this->error(__('Parameter %s can not be empty', ''));
        }
        $params = $this->preExcludeFields($params);

        if ($this->dataLimit && $this->dataLimitFieldAutoFill) {
            $params[$this->dataLimitField] = $this->auth->id;
        }
        $result = false;
        Db::startTrans();
        try {
            //是否采用模型验证
            if ($this->modelValidate) {
                $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.add' : $name) : $this->modelValidate;
                $this->model->validateFailException()->validate($validate);
            }
            //判断设备数量够不够
            $deviceAmount = $this->Chargedevicemodel->where(['user_id' => 0, 'authorizestatus' => 1, 'group_id' => $params['group_id']])->count('1');
            if ($params['amount'] > $deviceAmount) {
                Db::rollback();
                $this->error("当前选择小区可授权设备不足！只剩" . $deviceAmount);
            }
            $site = $this->model->where(['agent_user_id' => $params['agent_user_id'], 'group_id' => $params['group_id']])->find();
            if (!$site) {
                $deviceInfo = $this->Chargedevicemodel->where(['user_id' => 0, 'authorizestatus' => 1, 'group_id' => $params['group_id']])->find();
                $siteData = [
                    'agent_user_id' => $params['agent_user_id'],
                    'province_id' => $deviceInfo['province_id'],
                    'city_id' => $deviceInfo['city_id'],
                    'area_id' => $deviceInfo['area_id'],
                    'area_name' => $deviceInfo['area_name'],
                    'createtime' => time(),
                    'status' => 1,
                    'group_id' => $deviceInfo['group_id']
                ];
                $site = $this->model->create($siteData);
            }
            if (!$site) {
                $this->error("当前桩主未开通" . $deviceInfo['area_name'] . "站点权限,请手动开通");
            }
            $update_arr['user_id'] = $params['agent_user_id'];
            $update_arr['authorizestatus'] = 2;
            $update_arr['authorizetime'] = strtotime(date('Y-m-d')) + 86400;
            $update_arr['site_id'] = $site['id'];
            $this->Chargedevicemodel->where(['user_id' => 0, 'authorizestatus' => 1, 'group_id' => $params['group_id']])->limit($params['amount'])->update($update_arr);
            Db::commit();
            $this->success('操作成功');
        } catch (ValidateException | PDOException | Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($result === false) {
            $this->error(__('No rows were inserted'));
        }
        $this->success();
    }
}
